>From d5d0a61a19906d2a27fff6d6c9394fb51e2a5657 Mon Sep 17 00:00:00 2001
From: Roiy Shpaner <roiy@cs.umanitoba.ca>
Date: Tue, 15 Apr 2014 19:10:46 -0500
Subject: [PATCH 08/10] MED Support - Creation dialogs

---
 modules/gui/qt4/components/create_event.cpp  | 449 +++++++++++++++++++++++++++
 modules/gui/qt4/components/create_event.hpp  |  88 ++++++
 modules/gui/qt4/dialogs/event_properties.cpp | 156 ++++++++++
 modules/gui/qt4/dialogs/event_properties.hpp |  58 ++++
 modules/gui/qt4/dialogs/gototime.cpp         |   2 +-
 modules/gui/qt4/dialogs/new_layer.cpp        | 132 ++++++++
 modules/gui/qt4/dialogs/new_layer.hpp        |  55 ++++
 7 files changed, 939 insertions(+), 1 deletion(-)
 create mode 100644 modules/gui/qt4/components/create_event.cpp
 create mode 100644 modules/gui/qt4/components/create_event.hpp
 create mode 100644 modules/gui/qt4/dialogs/event_properties.cpp
 create mode 100644 modules/gui/qt4/dialogs/event_properties.hpp
 create mode 100644 modules/gui/qt4/dialogs/new_layer.cpp
 create mode 100644 modules/gui/qt4/dialogs/new_layer.hpp

diff --git a/modules/gui/qt4/components/create_event.cpp b/modules/gui/qt4/components/create_event.cpp
new file mode 100644
index 0000000..c4ef966
--- /dev/null
+++ b/modules/gui/qt4/components/create_event.cpp
@@ -0,0 +1,449 @@
+/*****************************************************************************
+ * create_event.cpp : Create Event Dialog
+ ****************************************************************************
+ * Copyright (C) 2014 the VideoLAN team
+ * $Id$
+ *
+ * Author: Roiy Shpaner <roiy@cs.umanitoba.ca>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include "qt4.hpp"
+#include "components/create_event.hpp"
+#include "dialogs/event_properties.hpp"
+#include "components/interface_widgets.hpp"
+#include "dialogs/new_layer.hpp"
+
+#include <QGridLayout>
+#include <QTimeEdit>
+#include <QDateTime>
+#include <QTime>
+#include <QTextEdit>
+#include <QComboBox>
+#include <QString>
+#include <QMessageBox>
+
+/**
+ * MED Event Creation tool
+ * MED Event properties are displayed and modified here
+ */
+CreateEventDialog::CreateEventDialog( QWidget *parent,
+                      intf_thread_t *_p_intf )
+                      : QWidget( parent ), p_intf( _p_intf )
+{
+    bIsEditingEnd = true;
+    bLayerAdded = false;
+
+    EventPropertiesLayout = new QGridLayout( this );
+    EventPropertiesLayout->setVerticalSpacing( 0 );
+
+    QFrame* timeFrame = new QFrame( this );
+    timeFrame->setFrameStyle( QFrame::Box );
+
+    timeEditLayout = new QGridLayout( timeFrame );
+
+    startTimeIntro = new QLabel( qtr( "Start Time:" ) );
+    startTimeIntro->setWordWrap( false );
+    startTimeIntro->setAlignment( Qt::AlignRight );
+    startTimeIntro->setStyleSheet( "font: bold" );  //for maximum size
+    int labWidth = startTimeIntro->sizeHint().width();
+    startTimeIntro->setFixedWidth( labWidth );
+    startTimeIntro->setStyleSheet( "" );
+
+    startTime = new QTimeEdit();
+    startTime->setDisplayFormat( "HH:mm:ss.zzz" );
+    startTime->setButtonSymbols( QAbstractSpinBox::NoButtons );
+    startTime->setAlignment( Qt::AlignLeft );
+    startTime->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
+    startTime->setReadOnly( true );
+    startTime->setEnabled( false );
+
+    focusStartButton = new QPushButton( QIcon(":/util/focus"), "" );
+    focusStartButton->setToolTip( qtr("Focus Start Time") );
+    CONNECT( focusStartButton, clicked( ),
+                 this, setEditModeStart( ) );
+
+    QHBoxLayout *startLayout = new QHBoxLayout( timeFrame );
+    startLayout->addWidget( startTimeIntro );
+    startLayout->addWidget( startTime );
+    startLayout->addWidget( focusStartButton );
+    startLayout->insertStretch( -1 );
+    timeEditLayout->addLayout( startLayout, 0, 0, 1, 3 );
+
+    /* End Time */
+
+    endTimeIntro = new QLabel( qtr( "End Time:" ) );
+    endTimeIntro->setWordWrap( false );
+    endTimeIntro->setAlignment( Qt::AlignRight );
+    endTimeIntro->setStyleSheet( "font: bold" );
+    endTimeIntro->setFixedWidth( labWidth );
+
+    endTime = new QTimeEdit();
+    endTime->setDisplayFormat( "HH:mm:ss.zzz" );
+    endTime->setButtonSymbols( QAbstractSpinBox::NoButtons );
+    endTime->setAlignment( Qt::AlignLeft );
+    endTime->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
+    endTime->setReadOnly( true );
+
+    focusEndButton = new QPushButton( QIcon(":/util/focus"), "" );
+    focusEndButton->setToolTip( qtr("Focus End Time") );
+    focusEndButton->setEnabled( false );
+    CONNECT( focusEndButton, clicked( ),
+                 this, setEditModeEnd( ) );
+
+    QHBoxLayout *endLayout = new QHBoxLayout( timeFrame );
+    endLayout->addWidget( endTimeIntro );
+    endLayout->addWidget( endTime );
+    endLayout->addWidget( focusEndButton );
+    endLayout->insertStretch( -1 );
+    timeEditLayout->addLayout( endLayout, 1, 0, 1, 3 );
+
+
+
+    QPushButton *setTimeButton = new QPushButton( QIcon(":/util/clock"), "" );
+    setTimeButton->setFixedSize( setTimeButton->sizeHint() );
+    setTimeButton->setToolTip( qtr("Set Time") );
+
+    CONNECT( setTimeButton, clicked( ),
+                 this, setTimeDialog( ) );
+
+    QFrame* line = new QFrame();
+    line->setFrameShape(QFrame::VLine);
+    line->setFrameShadow(QFrame::Sunken);
+
+    QLabel *jumpLabel = new QLabel( qtr( "Jump " ) );
+    jumpVal = new QSpinBox();
+    jumpVal->setRange( 1, 999 );
+    jumpVal->setValue( 5 );  /* default jump seconds value */
+    QLabel *jumpLabelEnd = new QLabel( qtr( " seconds " ) );
+
+    QPushButton *jumpBackButton = new QPushButton( QIcon(":/util/jump_back"), "" );
+    jumpBackButton->setToolTip( qtr("Jump back") );
+    CONNECT( jumpBackButton, clicked( ),
+                 this, jumpBack( ) );
+
+    QPushButton *jumpFwdButton = new QPushButton( QIcon(":/util/jump_forward"), "" );
+    jumpFwdButton->setToolTip( qtr("Jump forward") );
+    CONNECT( jumpFwdButton, clicked( ),
+                 this, jumpFwd( ) );
+
+    QHBoxLayout *jumpLayout = new QHBoxLayout( timeFrame );
+    jumpLayout->addWidget( setTimeButton );
+    jumpLayout->addWidget( line );
+    jumpLayout->addWidget( jumpLabel );
+    jumpLayout->addWidget( jumpVal );
+    jumpLayout->addWidget( jumpLabelEnd);
+    jumpLayout->addWidget( jumpBackButton );
+    jumpLayout->addWidget( jumpFwdButton );
+    jumpLayout->insertStretch(-1);
+    timeEditLayout->addLayout( jumpLayout, 3, 0, 1, 3 );
+
+    timeFrame->setLayout( timeEditLayout );
+    EventPropertiesLayout->addWidget( timeFrame, 0, 0, 4, 3 );
+
+    EventPropertiesLayout->setVerticalSpacing( 15 );
+
+    QLabel* label = new QLabel( qtr( N_("Events Layer: ") ) ); 
+    label->setContentsMargins( 3, 2, 0, 0 );
+    label->setAlignment( Qt::AlignRight );
+    EventPropertiesLayout->addWidget( label, 4, 0 );
+
+    layersCombo = new QComboBox( this );
+    layersCombo->setObjectName(QString::fromUtf8("layersCombo"));
+    QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+    sizePolicy.setHorizontalStretch(0);
+    sizePolicy.setVerticalStretch(0);
+    sizePolicy.setHeightForWidth(layersCombo->sizePolicy().hasHeightForWidth());
+    layersCombo->setSizePolicy(sizePolicy);
+
+    EventPropertiesLayout->addWidget(layersCombo, 4, 1);
+
+    QPushButton *newLayerButton = new QPushButton( qtr( "&New" ) );
+    newLayerButton->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
+    EventPropertiesLayout->addWidget(newLayerButton, 4, 2);
+
+    CONNECT( newLayerButton, clicked(), this, addLayer() );
+
+    EventPropertiesLayout->setVerticalSpacing( 5 );
+
+    label = new QLabel( qtr( N_("Annotation:") ) ); 
+    label->setContentsMargins( 3, 2, 0, 0 );
+    EventPropertiesLayout->addWidget( label, 5, 0, 1, 7 );
+    annotation_text = new QTextEdit;
+    annotation_text->setAcceptRichText( false );
+    EventPropertiesLayout->addWidget( annotation_text, 6, 0, 1, 7 );
+
+    CONNECT( THEMIM->getIM(), eventsChanged( bool ), this, update( bool ) );
+
+    newLayerDialog = NewLayerDialog::getInstance( p_intf );
+    CONNECT( newLayerDialog, layerAdded( QString ), this, layerAdded( QString ) );
+
+    update( true );
+}
+
+
+void CreateEventDialog::update( bool inputExists )
+{
+    clear();
+
+    if ( !inputExists )
+        return;
+
+    int* pi_layers;
+    char*** psz_layerNames;
+    if ( THEMIM->getIM()->getMedLayerNames( &psz_layerNames, &pi_layers ) != VLC_SUCCESS )
+        return;
+
+    layersCombo->addItem( qtr( "" ) );
+    for ( int i = 0; i < *pi_layers ; i++ )
+        layersCombo->addItem( qtr( (*psz_layerNames)[i] ) );
+}
+
+void CreateEventDialog::updateSegTimes( float pos, int64_t t, int64_t length )
+{
+    VLC_UNUSED( length );
+
+    if ( pos == -1.f )
+        return;
+    
+    bool bFirstSet = false; 
+
+    long time = t / 1000;
+    int ms = time % 1000;
+    
+    time /= 1000;
+    int seconds = time % 60;
+
+    time /= 60;
+    int minutes = time % 60;
+
+    time /= 60;
+    int hours = time;
+
+    QTime newTime;
+    newTime.setHMS( hours, minutes, seconds, ms );
+        
+    if ( ( startTime->time() == QTime( 0,0,0 ) ) && ( endTime->time() == QTime( 0,0,0 ) ) )
+        bFirstSet = true;   /* On first use only, we need both times as the current marker */
+
+    /* update time and prevent circular updating by not triggeting the signal */
+    if ( bIsEditingEnd )
+    {
+        endTime->setTime( newTime );
+
+        if ( endTime->time() < startTime->time() )  //Avoid end before start
+        {
+            startTime->setTime( newTime );
+            startTimeDisplayUpdate( newTime );
+        }
+
+    }
+    if ( !bIsEditingEnd || bFirstSet )
+    {
+        startTime->setTime( newTime );
+
+        if ( endTime->time() < startTime->time() )  //Avoid end before start
+        {
+            endTime->setTime( newTime );
+            endTimeDisplayUpdate( newTime );
+        }
+    }
+}
+
+/* sends playback commands according to the time set in the startTime */
+void CreateEventDialog::startTimeDisplayUpdate( QTime newTime )
+{
+    int64_t i_time = (int64_t) ( QTime( 0, 0, 0 ).msecsTo( newTime ) ) * 1000;
+
+    THEMIM->getIM()->setEventTimes( i_time, INT64_C(-1) );
+}
+
+/* sends playback commands according to the time set in the endTime */
+void CreateEventDialog::endTimeDisplayUpdate( QTime newTime )
+{
+    int64_t i_time = (int64_t) ( QTime( 0, 0, 0 ).msecsTo( newTime ) ) * 1000;
+
+    THEMIM->getIM()->setEventTimes( INT64_C(-1), i_time );
+}
+
+/* jump x seconds back */
+void CreateEventDialog::jumpBack( )
+{
+    jumpXSeconds( false );
+}
+
+/* jump x seconds forward */
+void CreateEventDialog::jumpFwd( )
+{
+    jumpXSeconds( true );
+}
+
+/* jump x seconds */
+void CreateEventDialog::jumpXSeconds( bool forwardJump )
+{
+    int64_t jumpLen = jumpVal->value() * 1000000;
+    int64_t jumpTime;
+
+    if ( bIsEditingEnd )
+        jumpTime = (int64_t) ( QTime( 0, 0, 0 ).msecsTo( endTime->time() ) ) * 1000;
+    else
+        jumpTime = (int64_t) ( QTime( 0, 0, 0 ).msecsTo( startTime->time() ) ) * 1000;
+
+    if ( forwardJump )
+        jumpTime += jumpLen;
+    else
+        jumpTime -= jumpLen;
+    
+    if ( jumpTime < 0 )
+        jumpTime = 0;
+
+    THEMIM->getIM()->setInputTime( jumpTime );   
+}
+
+/* Switches between editing the start or the end of the segment, with the last point where the marker was when swapped */
+void CreateEventDialog::setEditingMode( bool bIsSegEndEdit )
+{
+    bIsEditingEnd = bIsSegEndEdit;
+    
+    if ( bIsEditingEnd )
+    {
+        startTimeIntro->setStyleSheet( "" );
+        startTime->setEnabled( false );
+        focusStartButton->setEnabled( true );
+        endTimeIntro->setStyleSheet( "font: bold" );
+        endTime->setEnabled( true );
+        focusEndButton->setEnabled( false );
+    }
+    else
+    {
+        endTimeIntro->setStyleSheet( "" );
+        endTime->setEnabled( false );
+        focusEndButton->setEnabled( true );
+        startTimeIntro->setStyleSheet( "font: bold" );
+        startTime->setEnabled( true );
+        focusStartButton->setEnabled( false );
+    }
+}
+
+void CreateEventDialog::setEditModeStart( )
+{
+    setEditingMode( false );
+    THEMIM->getIM()->setMedEditMode( segmentStart );
+}
+    
+void CreateEventDialog::setEditModeEnd( )
+{
+    setEditingMode( true );
+    THEMIM->getIM()->setMedEditMode( segmentEnd );
+}
+
+void CreateEventDialog::setTimeDialog( )
+{
+    THEDP->gotoTimeDialog();
+}
+
+void CreateEventDialog::saveEvent()
+{
+    if ( layersCombo->currentIndex() == 0 )
+    {
+        QMessageBox msgBox;
+        msgBox.setText( "<p align='center'>Please pick an Events Layer</p>" );
+        msgBox.exec();
+        return;
+    }
+
+    /* Add layer if needed */
+
+    if ( bLayerAdded && ( layersCombo->currentIndex() == layersCombo->count() - 1 ) )
+    {
+
+        char* newLayer = strdup ( (char*) qtu( layersCombo->currentText() ) );
+
+        if ( THEMIM->getIM()->addMedLayer( &newLayer ) != VLC_SUCCESS )
+            return;
+
+        bLayerAdded = false;
+    }
+
+    int64_t startTimeSave = (int64_t) ( QTime( 0, 0, 0 ).msecsTo( startTime->time() ) ) * 1000;   
+    
+    int64_t endTimeSave = (int64_t) ( QTime( 0, 0, 0 ).msecsTo( endTime->time() ) ) * 1000;
+
+    //Build new event segment
+    seeksegment_t* newSegment = vlc_seeksegment_New();
+    newSegment->psz_layerName = strdup( (char*) qtu( layersCombo->currentText() ) );
+    newSegment->i_layer_id = layersCombo->currentIndex() - 1;
+    newSegment->i_time_offset = startTimeSave;
+    newSegment->i_time_duration = endTimeSave - startTimeSave;
+    char* newAnnotation = strdup( qtu( annotation_text->toPlainText() ) );
+    newSegment->psz_text = newAnnotation;
+
+
+    if ( THEMIM->getIM()->addMedSegment( newSegment ) != VLC_SUCCESS )
+    {
+        msg_Err( p_intf, "Could not create MED event!");
+        return;
+    }
+
+    vlc_seeksegment_Delete( newSegment );
+        
+    THEMIM->getIM()->eventsModified();
+
+    ( ( EventPropertiesDialog* ) parent() )->close();
+}
+
+void CreateEventDialog::addLayer()
+{
+    newLayerDialog->toggleVisible();
+}
+
+void CreateEventDialog::layerAdded( QString newLayer )
+{
+    if ( !isVisible() )  //function only useful when window is active
+        return;
+
+    if ( !bLayerAdded )
+        layersCombo->addItem( newLayer );
+    else
+        layersCombo->setItemText( layersCombo->count() - 1, newLayer );     //Just update the layer name to be added
+
+    layersCombo->setCurrentIndex( layersCombo->count() - 1 );
+
+    bLayerAdded = true;
+}
+
+void CreateEventDialog::clear()
+{
+    startTime->clear();
+    startTime->setTime( QTime( 0, 0, 0) );
+    endTime->clear();
+    endTime->setTime( QTime( 0, 0, 0) );
+    annotation_text->clear();
+    layersCombo->clear();
+    
+    setEditingMode( true );
+    bLayerAdded = false;
+}
+
+CreateEventDialog::~CreateEventDialog()
+{
+    clear();
+    delete EventPropertiesLayout;
+}
\ No newline at end of file
diff --git a/modules/gui/qt4/components/create_event.hpp b/modules/gui/qt4/components/create_event.hpp
new file mode 100644
index 0000000..6082457
--- /dev/null
+++ b/modules/gui/qt4/components/create_event.hpp
@@ -0,0 +1,88 @@
+/*****************************************************************************
+ * create_event.hpp : Create Event Dialog
+ ****************************************************************************
+ * Copyright (C) 2014 the VideoLAN team
+ * $Id$
+ *
+ * Author: Roiy Shpaner <roiy@cs.umanitoba.ca>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+#ifndef _CREATEEVENT_H_
+#define _CREATEEVENT_H_
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include "dialogs/new_layer.hpp"
+
+#include <QWidget>
+
+#include <vector>
+#include <QDateTime>
+
+class QTextEdit;
+class QTimeEdit;
+class QTime;
+class QLabel;
+class QComboBox;
+class QSpinBox;
+
+class CreateEventDialog: public QWidget
+{
+    Q_OBJECT
+public:
+    CreateEventDialog( QWidget *, struct intf_thread_t * );
+    ~CreateEventDialog();
+    QGridLayout *EventPropertiesLayout;
+    QGridLayout *timeEditLayout;
+    bool bIsEditingEnd;
+private:
+    void startTimeDisplayUpdate( QTime newTime );
+    void endTimeDisplayUpdate( QTime newTime );
+    void jumpXSeconds( bool forwardJump );
+
+    struct intf_thread_t *p_intf;
+    QLabel *startTimeIntro;
+    QTimeEdit *startTime;
+    QPushButton *focusStartButton;
+    QLabel *endTimeIntro;
+    QTimeEdit *endTime;
+    QPushButton *focusEndButton;
+    QSpinBox *jumpVal;
+    QTextEdit *annotation_text;
+    QComboBox *layersCombo;
+    NewLayerDialog * newLayerDialog;
+    bool bLayerAdded;
+
+public slots:
+    void update( bool inputExists );
+    void setEditingMode( bool bIsSegEndEdit );
+    void updateSegTimes( float pos, int64_t t, int64_t length );
+    void setEditModeStart();
+    void setEditModeEnd();
+    void setTimeDialog();
+    void jumpBack();
+    void jumpFwd();
+    void addLayer();
+    void layerAdded( QString newLayer );
+    void saveEvent();
+    void clear();
+};
+
+#endif
diff --git a/modules/gui/qt4/dialogs/event_properties.cpp b/modules/gui/qt4/dialogs/event_properties.cpp
new file mode 100644
index 0000000..25ffd11
--- /dev/null
+++ b/modules/gui/qt4/dialogs/event_properties.cpp
@@ -0,0 +1,156 @@
+/*****************************************************************************
+ * events_properties.cpp : events_properties display
+ ****************************************************************************
+ * Copyright (C) 2014 the VideoLAN team
+ * $Id$
+ *
+ * Authors: Roiy Shpaner <roiy@cs.umanitoba.ca>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include "dialogs/event_properties.hpp"
+
+#include "main_interface.hpp" /* Needed for external MI size */
+#include "input_manager.hpp"
+
+#include <QDialogButtonBox>
+#include <QPushButton>
+#include <vlc_modules.h>
+
+EventPropertiesDialog::EventPropertiesDialog( intf_thread_t *_p_intf )
+               : QVLCDialog( (QWidget*)_p_intf->p_sys->p_mi, _p_intf )
+{
+    bAutoJumpState = true;
+
+    setWindowOpacity( var_InheritFloat( p_intf, "qt-opacity" ) );
+    setWindowTitle( qtr( "Add a new Event" ) );
+    setWindowRole( "vlc-eventProperties" );
+
+    QVBoxLayout *layout = new QVBoxLayout( this );
+    layout->setContentsMargins( 0, 0, 0, 0 );
+    layout->setSpacing( 1 );
+
+    createEventDialog = new CreateEventDialog( this , p_intf );
+    layout->addWidget( createEventDialog , 0 , Qt::AlignCenter );
+
+    QDialogButtonBox *buttonBox = new QDialogButtonBox( Qt::Horizontal, this );
+    QPushButton *but = buttonBox->addButton( qtr("&Save"), QDialogButtonBox::AcceptRole );
+    CONNECT( but, clicked(), createEventDialog, saveEvent() );
+
+    but = buttonBox->addButton( qtr("&Close"), QDialogButtonBox::RejectRole );
+    CONNECT( but, clicked(), this, close() );
+
+    QHBoxLayout *buttonLayout = new QHBoxLayout( this );
+    buttonLayout->setSpacing( 0 );
+    buttonLayout->setContentsMargins( 0, 0, 0, 0 );
+    buttonLayout->addStretch( 1 );
+    buttonLayout->addWidget( buttonBox, 0 , Qt::AlignRight  );
+    layout->addLayout( buttonLayout );
+
+    CONNECT( THEMIM->getIM(), eventsChanged( bool ), this, update( bool ) );
+
+    /* Restore geometry or move this dialog on the left pane of the MI */
+    if( !restoreGeometry( getSettings()->value("EventPropertiesPanel/geometry").toByteArray() ) )
+    {
+        resize( QSize( 400, 100 ) );
+
+        MainInterface *p_mi = p_intf->p_sys->p_mi;
+        if( p_mi && p_mi->x() > 50 )
+            move( ( p_mi->x() - frameGeometry().width() - 10 ), p_mi->y() );
+        else
+            move ( 450 , 0 );
+    }
+}
+
+EventPropertiesDialog::~EventPropertiesDialog()
+{
+    getSettings()->setValue("EventPropertiesPanel/geometry", saveGeometry());
+}
+
+void EventPropertiesDialog::update( bool inputExists )
+{
+    if ( !inputExists )
+        close();
+}
+
+void EventPropertiesDialog::close()
+{
+    THEMIM->getIM()->setMedEditMode( disabled );
+    tmpDisableAutoJump( false );
+
+    disconnect( THEMIM->getIM(), SIGNAL( positionUpdated( float, int64_t, int64_t ) ),
+        createEventDialog, SLOT( updateSegTimes( float, int64_t, int64_t ) ) );
+    disconnect( THEMIM->getIM(), SIGNAL( switchUiEdit( bool ) ),
+        createEventDialog, SLOT( setEditingMode( bool ) ) );
+
+    QVLCDialog::close();
+}
+
+void EventPropertiesDialog::reject()
+{
+    close();
+    QDialog::reject();
+}
+
+void EventPropertiesDialog::show()
+{
+    createEventDialog->update( true );
+
+    THEMIM->getIM()->setMedEditMode( segmentEnd );
+    tmpDisableAutoJump( true );
+
+    int64_t curTime;
+    THEMIM->getIM()->getInputTime( &curTime );
+    createEventDialog->updateSegTimes( 0, curTime, 0 );
+
+    CONNECT( THEMIM->getIM(), positionUpdated( float, int64_t, int64_t ),
+        createEventDialog, updateSegTimes( float, int64_t, int64_t ) );
+    CONNECT( THEMIM->getIM(), switchUiEdit( bool ),
+        createEventDialog, setEditingMode( bool ) );
+
+    QVLCDialog::show();
+}
+
+void EventPropertiesDialog::tmpDisableAutoJump( bool bInEditMode )
+{
+    if ( bInEditMode )
+    {
+        THEMIM->getIM()->getAutoJump( &bAutoJumpState ); /* save autojump state */
+
+        THEMIM->getIM()->setAutoJump( false ); /* Set to off */
+
+        THEMIM->setAutoJumpButton( false );
+    }
+    else
+    {
+        THEMIM->getIM()->setAutoJump( bAutoJumpState ); /* restore autojump state */
+
+        THEMIM->setAutoJumpButton( true );
+    }
+}
+
+void EventPropertiesDialog::toggleVisible()
+{
+    if( isVisible() ) 
+        close();      //need the close function called...
+    else if ( THEMIM->getIM()->hasInput() )
+        show();
+}
+
+
diff --git a/modules/gui/qt4/dialogs/event_properties.hpp b/modules/gui/qt4/dialogs/event_properties.hpp
new file mode 100644
index 0000000..a6c1124
--- /dev/null
+++ b/modules/gui/qt4/dialogs/event_properties.hpp
@@ -0,0 +1,58 @@
+/*****************************************************************************
+ * events_properties.hpp : event properties dialogue
+ ****************************************************************************
+ * Copyright (C) 2014 the VideoLAN team
+ * $Id$
+ *
+ * Authors: Roiy Shpaner <roiy@cs.umanitoba.ca>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+#ifndef QVLC_EVENT_PROPERTIES_DIALOG_H_
+#define QVLC_EVENT_PROPERTIES_DIALOG_H_ 1
+
+#include "components/create_event.hpp"
+#include "util/qvlcframe.hpp"
+
+#include "util/singleton.hpp"
+
+class QTabWidget;
+class QTextEdit;
+
+class EventPropertiesDialog : public QVLCDialog, public Singleton<EventPropertiesDialog>
+{
+    Q_OBJECT
+private:
+    EventPropertiesDialog( intf_thread_t * );
+    virtual ~EventPropertiesDialog();
+
+    CreateEventDialog *createEventDialog;
+    void tmpDisableAutoJump( bool bInEditMode );
+    bool bAutoJumpState;
+
+private slots:
+    friend class    Singleton<EventPropertiesDialog>;
+    void update( bool inputExists );
+    void reject();
+
+public slots:
+    void close();
+    void show();
+    void toggleVisible();
+};
+
+#endif
+
diff --git a/modules/gui/qt4/dialogs/gototime.cpp b/modules/gui/qt4/dialogs/gototime.cpp
index 77035fb..5baef10 100644
--- a/modules/gui/qt4/dialogs/gototime.cpp
+++ b/modules/gui/qt4/dialogs/gototime.cpp
@@ -58,7 +58,7 @@ GotoTimeDialog::GotoTimeDialog( intf_thread_t *_p_intf)
     timeIntro->setAlignment( Qt::AlignCenter );
 
     timeEdit = new QTimeEdit();
-    timeEdit->setDisplayFormat( "HH'H':mm'm':ss's'" );
+    timeEdit->setDisplayFormat( "HH'H':mm'm':ss's'.zzz'ms'" );
     timeEdit->setAlignment( Qt::AlignRight );
     timeEdit->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum );
 
diff --git a/modules/gui/qt4/dialogs/new_layer.cpp b/modules/gui/qt4/dialogs/new_layer.cpp
new file mode 100644
index 0000000..3b3aeaa
--- /dev/null
+++ b/modules/gui/qt4/dialogs/new_layer.cpp
@@ -0,0 +1,132 @@
+/*****************************************************************************
+ * new_layer.cpp : new_layer display
+ ****************************************************************************
+ * Copyright (C) 2014 the VideoLAN team
+ * $Id$
+ *
+ * Authors: Roiy Shpaner <roiy@cs.umanitoba.ca>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include "dialogs/new_layer.hpp"
+#include "input_manager.hpp"
+
+#include <QPushButton>
+#include <QLineEdit>
+#include <QLabel>
+#include <QDialogButtonBox>
+#include <QMessageBox>
+
+
+NewLayerDialog::NewLayerDialog( intf_thread_t *_p_intf )
+               : QVLCDialog( (QWidget*)_p_intf->p_sys->p_mi, _p_intf )
+{
+    setWindowOpacity( var_InheritFloat( p_intf, "qt-opacity" ) );
+    setWindowTitle( qtr( "Add New Events Layer" ) );
+    setWindowRole( "vlc-newLayer" );
+
+    QGridLayout *mainLayout = new QGridLayout( this );
+    mainLayout->setContentsMargins( 1, 1, 1, 1 );
+    mainLayout->setSpacing( 2 );
+    mainLayout->setSizeConstraint( QLayout::SetFixedSize );
+
+    QLabel *addIntro = new QLabel( qtr( "Add New Layer:" ) );
+    addIntro->setAlignment( Qt::AlignCenter );
+    
+    mainLayout->addWidget( addIntro, 0, 0, 1, 1 );
+    
+    layerName = new QLineEdit();
+
+    mainLayout->addWidget( layerName, 0, 1, 1, 1 );
+
+
+    QDialogButtonBox *saveButtonBox = new QDialogButtonBox( Qt::Horizontal, this );
+    saveButtonBox->addButton(
+        new QPushButton( qtr("&Add"), this ), QDialogButtonBox::AcceptRole );
+
+    QDialogButtonBox *closeButtonBox = new QDialogButtonBox( Qt::Horizontal, this );
+    closeButtonBox->addButton(
+        new QPushButton( qtr("&Cancel"), this ), QDialogButtonBox::RejectRole );
+
+    QHBoxLayout *buttonLayout = new QHBoxLayout( this );
+    buttonLayout->setSpacing( 0 );
+    buttonLayout->setContentsMargins( 0, 0, 0, 0 );
+    buttonLayout->addStretch( 1 );
+    buttonLayout->addWidget( saveButtonBox, 0 , Qt::AlignRight  );
+    buttonLayout->addWidget( closeButtonBox, 0 , Qt::AlignRight  );
+    mainLayout->addLayout( buttonLayout, 1, 0, 1, 2 );
+
+    CONNECT( closeButtonBox, rejected(), this, close() );
+    CONNECT( saveButtonBox, accepted(), this, addLayer() );
+
+    QVLCTools::restoreWidgetPosition( p_intf, "NewLayerDialog", this );
+
+    layerName->setFocus();
+}
+
+void NewLayerDialog::toggleVisible()
+{
+    QVLCDialog::toggleVisible();
+    layerName->setFocus();
+}
+
+void NewLayerDialog::close()
+{
+    layerName->clear();
+    toggleVisible();
+}
+
+NewLayerDialog::~NewLayerDialog()
+{
+    QVLCTools::saveWidgetPosition( p_intf, "NewLayerDialog", this );
+}
+
+void NewLayerDialog::addLayer()
+{
+    if ( layerName->text().isEmpty() )
+    {
+        QMessageBox msgBox;
+        msgBox.setText( "<p align='center'>Please enter a name for the new Layer</p>" );
+        msgBox.exec();
+        return;
+    }
+
+    int* pi_layers;
+    char*** psz_layerNames;
+    if ( THEMIM->getIM()->getMedLayerNames( &psz_layerNames, &pi_layers ) != VLC_SUCCESS )
+        return;
+
+    const char* newLayer = (const char*) qtu( layerName->text() );
+
+    for ( int i = 0 ; i < *pi_layers ; i++ )
+    {
+        if ( strnicmp( (*psz_layerNames)[i], newLayer, 255 ) == 0 )  /* Layer found, no need to add */
+        {
+            QMessageBox msgBox;
+            msgBox.setText( "<p align='center'>This layer name already exists.</p>" );
+            msgBox.exec();
+
+            close();
+            return;
+        }
+    }
+
+    emit layerAdded( layerName->text() );
+    close();
+}
\ No newline at end of file
diff --git a/modules/gui/qt4/dialogs/new_layer.hpp b/modules/gui/qt4/dialogs/new_layer.hpp
new file mode 100644
index 0000000..ce0f752
--- /dev/null
+++ b/modules/gui/qt4/dialogs/new_layer.hpp
@@ -0,0 +1,55 @@
+/*****************************************************************************
+ * new_layer.hpp : New layer name
+ ****************************************************************************
+ * Copyright (C) 2014 the VideoLAN team
+ * $Id$
+ *
+ * Authors: Roiy Shpaner <roiy@cs.umanitoba.ca>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+#ifndef QVLC_NEW_LAYER_DIALOG_H_
+#define QVLC_NEW_LAYER_DIALOG_H_ 1
+
+#include "util/qvlcframe.hpp"
+
+#include "util/singleton.hpp"
+
+class QLineEdit;
+
+class NewLayerDialog : public QVLCDialog, public Singleton<NewLayerDialog>
+{
+    Q_OBJECT
+private:
+    NewLayerDialog( intf_thread_t * );
+    virtual ~NewLayerDialog();
+    QWidget* _parent;
+    QLineEdit* layerName;
+
+private slots:
+    friend class    Singleton<NewLayerDialog>;
+    void close();
+
+signals:
+    void layerAdded( QString );
+
+public slots:
+    void addLayer();
+    void toggleVisible();
+};
+
+#endif
+
-- 
1.8.4.msysgit.0

